Package: SNPRelate
Type: Package
Title: Parallel Computing Toolset for Relatedness and Principal
        Component Analysis of SNP Data
Version: 1.45.0
Date: 2025-09-26
Depends: R (>= 2.15), gdsfmt (>= 1.8.3)
LinkingTo: gdsfmt
Imports: methods, RhpcBLASctl
Suggests: parallel, Matrix, RUnit, knitr, markdown, rmarkdown, MASS,
        BiocGenerics
Enhances: SeqArray (>= 1.12.0)
Authors@R: c(person("Xiuwen", "Zheng", role=c("aut", "cre", "cph"),
        email="zhengx@u.washington.edu", comment=c(ORCID="0000-0002-1390-0708")),
        person("Stephanie", "Gogarten", role="ctb", email="sdmorris@uw.edu"),
        person("Cathy", "Laurie", role="ctb", email="cclaurie@uw.edu"),
        person("Bruce", "Weir", role=c("ctb", "ths"), email="bsweir@uw.edu",
        comment=c(ORCID="0000-0002-4883-1247")))
Description: Genome-wide association studies (GWAS) are widely used to
        investigate the genetic basis of diseases and traits, but they pose many
        computational challenges. We developed an R package SNPRelate to provide
        a binary format for single-nucleotide polymorphism (SNP) data in GWAS
        utilizing CoreArray Genomic Data Structure (GDS) data files. The GDS
        format offers the efficient operations specifically designed for
        integers with two bits, since a SNP could occupy only two bits.
        SNPRelate is also designed to accelerate two key computations on SNP
        data using parallel computing for multi-core symmetric multiprocessing
        computer architectures: Principal Component Analysis (PCA) and
        relatedness analysis using Identity-By-Descent measures. The SNP GDS
        format is also used by the GWASTools package with the support of S4
        classes and generic functions. The extended GDS format is implemented
        in the SeqArray package to support the storage of single nucleotide
        variations (SNVs), insertion/deletion polymorphism (indel) and
        structural variation calls in whole-genome and whole-exome variant data.
License: GPL-3
VignetteBuilder: knitr
LazyData: true
URL: https://github.com/zhengxwen/SNPRelate
BugReports: https://github.com/zhengxwen/SNPRelate/issues
biocViews: Infrastructure, Genetics, StatisticalMethod,
        PrincipalComponent
git_url: https://git.bioconductor.org/packages/SNPRelate
git_branch: devel
git_last_commit: 3bd5f10
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: yes
Packaged: 2025-10-30 23:08:26 UTC; biocbuild
Author: Xiuwen Zheng [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-1390-0708>),
  Stephanie Gogarten [ctb],
  Cathy Laurie [ctb],
  Bruce Weir [ctb, ths] (ORCID: <https://orcid.org/0000-0002-4883-1247>)
Maintainer: Xiuwen Zheng <zhengx@u.washington.edu>
