\name{snpgdsErrMsg}
\alias{snpgdsErrMsg}
\title{
    Get the last error information
}
\description{
    Return the last error message.
}
\usage{
snpgdsErrMsg()
}
\value{Characters}

\author{Xiuwen Zheng}

\examples{
snpgdsErrMsg()
}

\keyword{GDS}
\keyword{GWAS}
